/**
  ******************************************************************************
  * @file           : i2c.c
  * @brief          : I2C base functions
  ******************************************************************************
  */

#include "main.h"

/* Private macros */
#define ENABLE_TIM  TIM17->CNT=1; TIM17->CR1 |= TIM_CR1_CEN
#define TIM_CNT     TIM17->CNT

/* Private variables */
static i2c_status_t st_Read = I2C_Ret_OK;

/* Private function prototypes */
i2c_status_t i2c_check_err(void);

/**
 * @brief Check I2C bus for errors.
 * @retval I2C return code
 */
i2c_status_t i2c_check_err(void) {
  i2c_status_t r = I2C_Ret_OK;

  if ((I2C1->ISR & I2C_ISR_NACKF) != 0) {
  /* device not present */
    r = I2C_Ret_NACK;
  } else if ((I2C1->ISR & I2C_ISR_TIMEOUT) != 0) {
  /* timeout error */
    r = I2C_Ret_Tout;
  } else if ((I2C1->ISR & (I2C_ISR_ARLO | I2C_ISR_BERR)) != 0) {
  /* other error */
    r = I2C_Ret_Err;
  }

  if (r != I2C_Ret_OK) {
  /* restart I2C and clear flags */
    I2C1->CR1 &= ~I2C_CR1_PE;
    while ((I2C1->CR1 & I2C_CR1_PE) != 0);
    I2C1->CR1 |= I2C_CR1_PE;
  }

  return r;
}

/**
 * @brief Read len bytes from I2C bus to data by reg_addr.
 * @retval I2C return code
 */
i2c_status_t user_i2c_read(const uint8_t id, const uint8_t reg_addr, uint8_t *data, const uint16_t len) {
  /* wait for i2c */
  while ( I2C1->ISR & I2C_ISR_BUSY ) { __NOP(); };

  st_Read = I2C_Ret_OK;

  Flag.I2C_RX_End = 0;
  Flag.I2C_RX_Err = 0;
  Flag.I2C_TX_Err = 0;

  /* prepare i2c for sending reg addr */
  I2C1->CR2 &= ~(I2C_CR2_SADD | I2C_CR2_NBYTES | I2C_CR2_RD_WRN);
  I2C1->CR2 |= (id | 1 << I2C_CR2_NBYTES_Pos);
  /* gen START */
  I2C1->CR2 |= I2C_CR2_START;

  /* wait for byte request or any error */
  //ENABLE_TIM;
  while ((I2C1->ISR & (I2C_ISR_TIMEOUT | I2C_ISR_ARLO | I2C_ISR_BERR | I2C_ISR_NACKF | I2C_ISR_TXE)) == 0) {
    //if (TIM_CNT == 0) {
    //  return I2C_Ret_Tout;
    //}
    __NOP();
  }

  if ((I2C1->ISR & I2C_ISR_TXE) != 0) {
  /* device ok, send reg addr */
    I2C1->TXDR = reg_addr;
  } else {
    st_Read = i2c_check_err();
    if (st_Read != I2C_Ret_OK) {
      Flag.I2C_TX_Err = 1;
      return st_Read;
    }
  }

  /* wait for i2c or any error */
  while (((I2C1->ISR & I2C_ISR_BUSY) != 0) && ((I2C1->ISR & (I2C_ISR_ARLO | I2C_ISR_BERR | I2C_ISR_NACKF)) == 0)) { __NOP(); };
  st_Read = i2c_check_err();
  if (st_Read != I2C_Ret_OK) {
    Flag.I2C_TX_Err = 1;
    return st_Read;
  }

  int cnt = 0;
  // Read data while no TC flag set.
  // Если взлетит NACK-флаг, приём прекращаем.
  //ENABLE_TIM;
  while ((((I2C1->ISR & I2C_ISR_TC)==0) && ((I2C1->ISR & I2C_ISR_NACKF)==0)) && (I2C1->ISR & I2C_ISR_BUSY)) {
    if (I2C1->ISR & I2C_ISR_RXNE) { // Reading data
      *data = I2C1->RXDR;
      data ++;
      cnt ++;
      if (cnt > len) {
        st_Read = I2C_Ret_Err;
        break;
      }
    }
    //if (TIM_CNT == 0) {
    //  st_Read = I2C_Ret_Tout;
    //  break;
    //}
  }

  /* wait for receiving data */
  //while ((Flag.I2C_RX_End == 0) && (Flag.I2C_RX_Err == 0)) { __NOP(); };

  /* Set STOP */
  //I2C1->CR2 |= I2C_CR2_STOP;
  /* Clear flags */
  //I2C1->ICR = (I2C_ICR_STOPCF | I2C_ICR_NACKCF | I2C_ICR_ARLOCF | I2C_ICR_BERRCF);

  return st_Read;
}

/**
 * @brief Write len bytes to I2C bus from data by reg_addr.
 * @retval I2C return code
 */
i2c_status_t user_i2c_write(const uint8_t id, const uint8_t reg_addr, uint8_t *data, const uint16_t len) {
  i2c_status_t r = I2C_Ret_OK;

  while ( I2C1->ISR & I2C_ISR_BUSY );

  Flag.I2C_TX_End = 0;
  Flag.I2C_TX_Err = 0;

  DMA1_Channel2->CMAR = (uint32_t)data;
  DMA1_Channel2->CPAR = (uint32_t)&(I2C1->TXDR);
  DMA1_Channel2->CNDTR = len;

  I2C1->CR2 &= ~( I2C_CR2_SADD | I2C_CR2_NBYTES | I2C_CR2_RD_WRN);
  I2C1->CR2 |= ( id | (len + 1) << I2C_CR2_NBYTES_Pos );
  I2C1->CR2 |= ( I2C_CR2_START );

  //ENABLE_TIM;
  while ((I2C1->ISR & (I2C_ISR_ARLO | I2C_ISR_BERR | I2C_ISR_NACKF | I2C_ISR_TXE | I2C_ISR_TIMEOUT)) == 0) {
    //if (TIM_CNT == 0) {
    //  return I2C_Ret_Tout;
    //}
    __NOP();
  }

  if ((I2C1->ISR & I2C_ISR_TXE) != 0) {
    I2C1->TXDR = reg_addr;
  } else {
    r = i2c_check_err();
    if (r != I2C_Ret_OK) {
      Flag.I2C_TX_Err = 1;
      return r;
    }
  }

  DMA1_Channel2->CCR |= DMA_CCR_EN;
  I2C1->CR1 |= I2C_CR1_TXDMAEN;

  return r;
}
